@@ -38,6 +38,7 @@
 #include <limits>
 #include <cerrno>
 #include <cstring>
+#include <cstdio>
 
 #include <gta/gta.h>
 
@@ -147,6 +148,15 @@ namespace gta
     {
         none = GTA_NONE,   /**< \brief No compression */
         zlib = GTA_ZLIB,   /**< \brief ZLIB compression (fast, moderate compression rate) */
+        zlib1 = GTA_ZLIB1, /**< \brief ZLIB compression with level 1 */
+        zlib2 = GTA_ZLIB2, /**< \brief ZLIB compression with level 2 */
+        zlib3 = GTA_ZLIB3, /**< \brief ZLIB compression with level 3 */
+        zlib4 = GTA_ZLIB4, /**< \brief ZLIB compression with level 4 */
+        zlib5 = GTA_ZLIB5, /**< \brief ZLIB compression with level 5 */
+        zlib6 = GTA_ZLIB6, /**< \brief ZLIB compression with level 6 */
+        zlib7 = GTA_ZLIB7, /**< \brief ZLIB compression with level 7 */
+        zlib8 = GTA_ZLIB8, /**< \brief ZLIB compression with level 8 */
+        zlib9 = GTA_ZLIB9, /**< \brief ZLIB compression with level 9 */
         bzip2 = GTA_BZIP2, /**< \brief BZIP2 compression (moderate speed, good compression rates) */
         xz = GTA_XZ        /**< \brief XZ compression (low/moderate speed, good/very good compression rates) */
     };
@@ -162,60 +172,44 @@ namespace gta
     {
     private:
 
-        static const int _whatsize = 64;
+        static const int _whatsize = 96;
         gta::result _r;
         int _sys_errno;
         char _what[_whatsize];
 
-    private:
-
-        void ctor(gta::result r) throw ()
-        {
-            _r = r;
-            _sys_errno = 0;
-            switch (r)
-            {
-            case ok:
-                strncpy(_what, "Success", _whatsize);
-                break;
-            case overflow:
-                strncpy(_what, "Value too large for data type", _whatsize);
-                break;
-            case unsupported_data:
-                strncpy(_what, "Unsupported data", _whatsize);
-                break;
-            case unexpected_eof:
-                strncpy(_what, "Unexpected end of input", _whatsize);
-                break;
-            case invalid_data:
-                strncpy(_what, "Invalid data", _whatsize);
-                break;
-            case system_error:
-                _sys_errno = errno;
-                strncpy(_what, strerror(_sys_errno), _whatsize);
-                break;
-            }
-            _what[_whatsize - 1] = '\0';
-        }
-
     public:
 
         /**
          * \brief       Constructor.
+         * \param s     The action that failed.
          * \param r     The GTA result.
          */
-        exception(gta::result r) throw ()
+        exception(const char *s, gta::result r) throw ()
+            : _r(r), _sys_errno(r == system_error ? errno : 0)
         {
-            ctor(r);
-        }
-
-        /**
-         * \brief       Constructor.
-         * \param r     The GTA result.
-         */
-        exception(gta_result_t r) throw ()
-        {
-            ctor(static_cast<gta::result>(r));
+            const char *w = "";
+            switch (r)
+            {
+            case ok:
+                w = "success";
+                break;
+            case overflow:
+                w = "value too large for data type";
+                break;
+            case unsupported_data:
+                w = "unsupported data";
+                break;
+            case unexpected_eof:
+                w = "unexpected end of input";
+                break;
+            case invalid_data:
+                w = "invalid data";
+                break;
+            case system_error:
+                w = strerror(_sys_errno);
+                break;
+            }
+            snprintf(_what, _whatsize, "%s: %s", s, w);
         }
 
         /**
@@ -280,7 +274,7 @@ namespace gta
             gta_result_t r = gta_clone_taglist(_taglist, taglist._taglist);
             if (r != GTA_OK)
             {
-                throw exception(r);
+                throw exception("cannot clone GTA tag list", static_cast<gta::result>(r));
             }
         }
         /** \endcond */
@@ -338,7 +332,7 @@ namespace gta
             gta_result_t r = gta_set_tag(_taglist, name, value);
             if (r != GTA_OK)
             {
-                throw exception(r);
+                throw exception("cannot set GTA tag", static_cast<gta::result>(r));
             }
         }
 
@@ -353,7 +347,7 @@ namespace gta
             gta_result_t r = gta_unset_tag(_taglist, name);
             if (r != GTA_OK)
             {
-                throw exception(r);
+                throw exception("cannot unset GTA tag", static_cast<gta::result>(r));
             }
         }
 
@@ -376,7 +370,7 @@ namespace gta
             gta_result_t r = gta_clone_taglist(_taglist, tl._taglist);
             if (r != GTA_OK)
             {
-                throw exception(r);
+                throw exception("cannot clone GTA taglist", static_cast<gta::result>(r));
             }
             return *this;
         }
@@ -655,7 +649,7 @@ namespace gta
             gta_result_t r = gta_init_io_state(&_state);
             if (r != GTA_OK)
             {
-                throw exception(r);
+                throw exception("cannot initialize GTA i/o state", static_cast<gta::result>(r));
             }
         }
 
@@ -665,12 +659,12 @@ namespace gta
             gta_result_t r = gta_init_io_state(&_state);
             if (r != GTA_OK)
             {
-                throw exception(r);
+                throw exception("cannot initialize GTA i/o state", static_cast<gta::result>(r));
             }
             r = gta_clone_io_state(_state, s._state);
             if (r != GTA_OK)
             {
-                throw exception(r);
+                throw exception("cannot clone GTA i/o state", static_cast<gta::result>(r));
             }
         }
         /** \endcond */
@@ -689,7 +683,7 @@ namespace gta
             gta_result_t r = gta_clone_io_state(_state, s._state);
             if (r != GTA_OK)
             {
-                throw exception(r);
+                throw exception("cannot clone GTA i/o state", static_cast<gta::result>(r));
             }
             return *this;
         }
@@ -730,7 +724,7 @@ namespace gta
             gta_result_t r = gta_init_header(&_header);
             if (r != GTA_OK)
             {
-                throw exception(r);
+                throw exception("cannot initialize GTA header", static_cast<gta::result>(r));
             }
         }
 
@@ -745,12 +739,12 @@ namespace gta
             r = gta_init_header(&_header);
             if (r != GTA_OK)
             {
-                throw exception(r);
+                throw exception("cannot initialize GTA header", static_cast<gta::result>(r));
             }
             r = gta_clone_header(_header, hdr._header);
             if (r != GTA_OK)
             {
-                throw exception(r);
+                throw exception("cannot clone GTA header", static_cast<gta::result>(r));
             }
         }
 
@@ -774,7 +768,7 @@ namespace gta
             gta_result_t r = gta_clone_header(_header, hdr._header);
             if (r != GTA_OK)
             {
-                throw exception(r);
+                throw exception("cannot clone GTA header", static_cast<gta::result>(r));
             }
             return *this;
         }
@@ -796,7 +790,7 @@ namespace gta
             gta_result_t r = gta_read_header(_header, read_custom_io, reinterpret_cast<intptr_t>(&io));
             if (r != GTA_OK)
             {
-                throw exception(r);
+                throw exception("cannot read GTA header", static_cast<gta::result>(r));
             }
         }
 
@@ -810,7 +804,7 @@ namespace gta
             gta_result_t r = gta_read_header(_header, read_custom_io, reinterpret_cast<intptr_t>(&io));
             if (r != GTA_OK)
             {
-                throw exception(r);
+                throw exception("cannot read GTA header", static_cast<gta::result>(r));
             }
         }
 
@@ -823,7 +817,7 @@ namespace gta
             gta_result_t r = gta_read_header_from_stream(_header, f);
             if (r != GTA_OK)
             {
-                throw exception(r);
+                throw exception("cannot read GTA header", static_cast<gta::result>(r));
             }
         }
 
@@ -836,7 +830,7 @@ namespace gta
             gta_result_t r = gta_read_header_from_fd(_header, fd);
             if (r != GTA_OK)
             {
-                throw exception(r);
+                throw exception("cannot read GTA header", static_cast<gta::result>(r));
             }
         }
 
@@ -849,7 +843,7 @@ namespace gta
             gta_result_t r = gta_write_header(_header, write_custom_io, reinterpret_cast<intptr_t>(&io));
             if (r != GTA_OK)
             {
-                throw exception(r);
+                throw exception("cannot write GTA header", static_cast<gta::result>(r));
             }
         }
 
@@ -863,7 +857,7 @@ namespace gta
             gta_result_t r = gta_write_header(_header, write_custom_io, reinterpret_cast<intptr_t>(&io));
             if (r != GTA_OK)
             {
-                throw exception(r);
+                throw exception("cannot write GTA header", static_cast<gta::result>(r));
             }
         }
 
@@ -876,7 +870,7 @@ namespace gta
             gta_result_t r = gta_write_header_to_stream(_header, f);
             if (r != GTA_OK)
             {
-                throw exception(r);
+                throw exception("cannot write GTA header", static_cast<gta::result>(r));
             }
         }
 
@@ -889,7 +883,7 @@ namespace gta
             gta_result_t r = gta_write_header_to_fd(_header, fd);
             if (r != GTA_OK)
             {
-                throw exception(r);
+                throw exception("cannot write GTA header", static_cast<gta::result>(r));
             }
         }
 
@@ -1038,15 +1032,28 @@ namespace gta
          * \brief               Check wether input data is stored in chunks.
          * \return              true or false.
          *
-         * Returns true is the array data is stored in chunks, which means that it
-         * cannot be accessed by the out-of-core input functions. If the data is chunked,
-         * it is most probably compressed.
+         * Returns true if the array data is stored in chunks, which means that it
+         * cannot be accessed by the out-of-core input functions.\n
+         * Note that compressed data is always stored in chunk, while uncompressed
+         * data is usually not stored in chunks.
          */
         bool data_is_chunked() const throw ()
         {
             return gta_data_is_chunked(_header);
         }
 
+        /**
+         * \brief               Get the compression.
+         * \return              The compression type.
+         *
+         * Gets the compression type for the header and data.\n
+         * See \a gta_compression_t for more information on compression types.
+         */
+        gta::compression compression() const throw ()
+        {
+            return static_cast<gta::compression>(gta_get_compression(_header));
+        }
+
         /**
          * \brief               Set the compression.
          * \param compression   The compression type.
@@ -1054,7 +1061,7 @@ namespace gta
          * Sets the compression type for writing the header and data.\n
          * See \a gta_compression_t for more information on compression types.
          */
-        void set_compression(compression compression) throw ()
+        void set_compression(gta::compression compression) throw ()
         {
             gta_set_compression(_header, static_cast<gta_compression_t>(compression));
         }
@@ -1085,7 +1092,7 @@ namespace gta
             gta_result_t r = gta_set_components(_header, n, reinterpret_cast<const gta_type_t *>(types), sizes);
             if (r != GTA_OK)
             {
-                throw exception(r);
+                throw exception("cannot set GTA components", static_cast<gta::result>(r));
             }
         }
 
@@ -1101,7 +1108,7 @@ namespace gta
             gta_result_t r = gta_set_components(_header, 1, reinterpret_cast<gta_type_t *>(types), sizes);
             if (r != GTA_OK)
             {
-                throw exception(r);
+                throw exception("cannot set GTA components", static_cast<gta::result>(r));
             }
         }
 
@@ -1120,7 +1127,7 @@ namespace gta
             gta_result_t r = gta_set_components(_header, 2, reinterpret_cast<gta_type_t *>(types), sizes);
             if (r != GTA_OK)
             {
-                throw exception(r);
+                throw exception("cannot set GTA components", static_cast<gta::result>(r));
             }
         }
 
@@ -1141,7 +1148,7 @@ namespace gta
             gta_result_t r = gta_set_components(_header, 3, reinterpret_cast<gta_type_t *>(types), sizes);
             if (r != GTA_OK)
             {
-                throw exception(r);
+                throw exception("cannot set GTA components", static_cast<gta::result>(r));
             }
         }
 
@@ -1164,7 +1171,7 @@ namespace gta
             gta_result_t r = gta_set_components(_header, 4, reinterpret_cast<gta_type_t *>(types), sizes);
             if (r != GTA_OK)
             {
-                throw exception(r);
+                throw exception("cannot set GTA components", static_cast<gta::result>(r));
             }
         }
 
@@ -1181,7 +1188,7 @@ namespace gta
             gta_result_t r = gta_set_dimensions(_header, n, sizes);
             if (r != GTA_OK)
             {
-                throw exception(r);
+                throw exception("cannot set GTA dimensions", static_cast<gta::result>(r));
             }
         }
 
@@ -1195,7 +1202,7 @@ namespace gta
             gta_result_t r = gta_set_dimensions(_header, 1, sizes);
             if (r != GTA_OK)
             {
-                throw exception(r);
+                throw exception("cannot set GTA dimensions", static_cast<gta::result>(r));
             }
         }
 
@@ -1210,7 +1217,7 @@ namespace gta
             gta_result_t r = gta_set_dimensions(_header, 2, sizes);
             if (r != GTA_OK)
             {
-                throw exception(r);
+                throw exception("cannot set GTA dimensions", static_cast<gta::result>(r));
             }
         }
 
@@ -1226,7 +1233,7 @@ namespace gta
             gta_result_t r = gta_set_dimensions(_header, 3, sizes);
             if (r != GTA_OK)
             {
-                throw exception(r);
+                throw exception("cannot set GTA dimensions", static_cast<gta::result>(r));
             }
         }
 
@@ -1243,7 +1250,7 @@ namespace gta
             gta_result_t r = gta_set_dimensions(_header, 4, sizes);
             if (r != GTA_OK)
             {
-                throw exception(r);
+                throw exception("cannot set GTA dimensions", static_cast<gta::result>(r));
             }
         }
 
@@ -1267,7 +1274,7 @@ namespace gta
             gta_result_t r = gta_read_data(_header, data, read_custom_io, reinterpret_cast<intptr_t>(&io));
             if (r != GTA_OK)
             {
-                throw exception(r);
+                throw exception("cannot read GTA data", static_cast<gta::result>(r));
             }
         }
 
@@ -1284,7 +1291,7 @@ namespace gta
             gta_result_t r = gta_read_data(_header, data, read_custom_io, reinterpret_cast<intptr_t>(&io));
             if (r != GTA_OK)
             {
-                throw exception(r);
+                throw exception("cannot read GTA data", static_cast<gta::result>(r));
             }
         }
 
@@ -1300,7 +1307,7 @@ namespace gta
             gta_result_t r = gta_read_data_from_stream(_header, data, f);
             if (r != GTA_OK)
             {
-                throw exception(r);
+                throw exception("cannot read GTA data", static_cast<gta::result>(r));
             }
         }
 
@@ -1316,7 +1323,7 @@ namespace gta
             gta_result_t r = gta_read_data_from_fd(_header, data, fd);
             if (r != GTA_OK)
             {
-                throw exception(r);
+                throw exception("cannot read GTA data", static_cast<gta::result>(r));
             }
         }
 
@@ -1333,7 +1340,7 @@ namespace gta
                     reinterpret_cast<intptr_t>(&io));
             if (r != GTA_OK)
             {
-                throw exception(r);
+                throw exception("cannot skip GTA data", static_cast<gta::result>(r));
             }
         }
 
@@ -1351,7 +1358,7 @@ namespace gta
                     reinterpret_cast<intptr_t>(&io));
             if (r != GTA_OK)
             {
-                throw exception(r);
+                throw exception("cannot skip GTA data", static_cast<gta::result>(r));
             }
         }
 
@@ -1366,7 +1373,7 @@ namespace gta
             gta_result_t r = gta_skip_data_from_stream(_header, f);
             if (r != GTA_OK)
             {
-                throw exception(r);
+                throw exception("cannot skip GTA data", static_cast<gta::result>(r));
             }
         }
 
@@ -1381,7 +1388,7 @@ namespace gta
             gta_result_t r = gta_skip_data_from_fd(_header, fd);
             if (r != GTA_OK)
             {
-                throw exception(r);
+                throw exception("cannot skip GTA data", static_cast<gta::result>(r));
             }
         }
 
@@ -1395,7 +1402,7 @@ namespace gta
             gta_result_t r = gta_write_data(_header, data, write_custom_io, reinterpret_cast<intptr_t>(&io));
             if (r != GTA_OK)
             {
-                throw exception(r);
+                throw exception("cannot write GTA data", static_cast<gta::result>(r));
             }
         }
 
@@ -1410,7 +1417,7 @@ namespace gta
             gta_result_t r = gta_write_data(_header, data, write_custom_io, reinterpret_cast<intptr_t>(&io));
             if (r != GTA_OK)
             {
-                throw exception(r);
+                throw exception("cannot write GTA data", static_cast<gta::result>(r));
             }
         }
 
@@ -1424,7 +1431,7 @@ namespace gta
             gta_result_t r = gta_write_data_to_stream(_header, data, f);
             if (r != GTA_OK)
             {
-                throw exception(r);
+                throw exception("cannot write GTA data", static_cast<gta::result>(r));
             }
         }
 
@@ -1438,7 +1445,7 @@ namespace gta
             gta_result_t r = gta_write_data_to_fd(_header, data, fd);
             if (r != GTA_OK)
             {
-                throw exception(r);
+                throw exception("cannot write GTA data", static_cast<gta::result>(r));
             }
         }
 
@@ -1456,7 +1463,7 @@ namespace gta
                     write_custom_io, reinterpret_cast<intptr_t>(&write_io));
             if (r != GTA_OK)
             {
-                throw exception(r);
+                throw exception("cannot copy GTA data", static_cast<gta::result>(r));
             }
         }
 
@@ -1476,7 +1483,7 @@ namespace gta
                     write_custom_io, reinterpret_cast<intptr_t>(&write_io));
             if (r != GTA_OK)
             {
-                throw exception(r);
+                throw exception("cannot copy GTA data", static_cast<gta::result>(r));
             }
         }
 
@@ -1491,7 +1498,7 @@ namespace gta
             gta_result_t r = gta_copy_data_stream(_header, fi, write_header._header, fo);
             if (r != GTA_OK)
             {
-                throw exception(r);
+                throw exception("cannot copy GTA data", static_cast<gta::result>(r));
             }
         }
 
@@ -1506,7 +1513,7 @@ namespace gta
             gta_result_t r = gta_copy_data_fd(_header, fdi, write_header._header, fdo);
             if (r != GTA_OK)
             {
-                throw exception(r);
+                throw exception("cannot copy GTA data", static_cast<gta::result>(r));
             }
         }
 
@@ -1707,7 +1714,7 @@ namespace gta
             gta_result_t r = gta_read_elements(_header, state._state, n, buf, read_custom_io, reinterpret_cast<intptr_t>(&io));
             if (r != GTA_OK)
             {
-                throw exception(r);
+                throw exception("cannot read GTA data elements", static_cast<gta::result>(r));
             }
         }
 
@@ -1726,7 +1733,7 @@ namespace gta
             gta_result_t r = gta_read_elements(_header, state._state, n, buf, read_custom_io, reinterpret_cast<intptr_t>(&io));
             if (r != GTA_OK)
             {
-                throw exception(r);
+                throw exception("cannot read GTA data elements", static_cast<gta::result>(r));
             }
         }
 
@@ -1744,7 +1751,7 @@ namespace gta
             gta_result_t r = gta_read_elements_from_stream(_header, state._state, n, buf, f);
             if (r != GTA_OK)
             {
-                throw exception(r);
+                throw exception("cannot read GTA data elements", static_cast<gta::result>(r));
             }
         }
 
@@ -1762,7 +1769,7 @@ namespace gta
             gta_result_t r = gta_read_elements_from_fd(_header, state._state, n, buf, fd);
             if (r != GTA_OK)
             {
-                throw exception(r);
+                throw exception("cannot read GTA data elements", static_cast<gta::result>(r));
             }
         }
 
@@ -1780,7 +1787,7 @@ namespace gta
             gta_result_t r = gta_write_elements(_header, state._state, n, buf, write_custom_io, reinterpret_cast<intptr_t>(&io));
             if (r != GTA_OK)
             {
-                throw exception(r);
+                throw exception("cannot write GTA data elements", static_cast<gta::result>(r));
             }
         }
 
@@ -1799,7 +1806,7 @@ namespace gta
             gta_result_t r = gta_write_elements(_header, state._state, n, buf, write_custom_io, reinterpret_cast<intptr_t>(&io));
             if (r != GTA_OK)
             {
-                throw exception(r);
+                throw exception("cannot write GTA data elements", static_cast<gta::result>(r));
             }
         }
 
@@ -1817,7 +1824,7 @@ namespace gta
             gta_result_t r = gta_write_elements_to_stream(_header, state._state, n, buf, f);
             if (r != GTA_OK)
             {
-                throw exception(r);
+                throw exception("cannot write GTA data elements", static_cast<gta::result>(r));
             }
         }
 
@@ -1835,7 +1842,7 @@ namespace gta
             gta_result_t r = gta_write_elements_to_fd(_header, state._state, n, buf, fd);
             if (r != GTA_OK)
             {
-                throw exception(r);
+                throw exception("cannot write GTA data elements", static_cast<gta::result>(r));
             }
         }
 
@@ -1876,7 +1883,7 @@ namespace gta
                     read_custom_io, seek_custom_io, reinterpret_cast<intptr_t>(&io));
             if (r != GTA_OK)
             {
-                throw exception(r);
+                throw exception("cannot read GTA data block", static_cast<gta::result>(r));
             }
         }
 
@@ -1901,7 +1908,7 @@ namespace gta
                     read_custom_io, seek_custom_io, reinterpret_cast<intptr_t>(&io));
             if (r != GTA_OK)
             {
-                throw exception(r);
+                throw exception("cannot read GTA data block", static_cast<gta::result>(r));
             }
         }
 
@@ -1924,7 +1931,7 @@ namespace gta
                     lower_coordinates, higher_coordinates, block, f);
             if (r != GTA_OK)
             {
-                throw exception(r);
+                throw exception("cannot read GTA data block", static_cast<gta::result>(r));
             }
         }
 
@@ -1947,7 +1954,7 @@ namespace gta
                     lower_coordinates, higher_coordinates, block, fd);
             if (r != GTA_OK)
             {
-                throw exception(r);
+                throw exception("cannot read GTA data block", static_cast<gta::result>(r));
             }
         }
 
@@ -1970,7 +1977,7 @@ namespace gta
                     write_custom_io, seek_custom_io, reinterpret_cast<intptr_t>(&io));
             if (r != GTA_OK)
             {
-                throw exception(r);
+                throw exception("cannot write GTA data block", static_cast<gta::result>(r));
             }
         }
 
@@ -1994,7 +2001,7 @@ namespace gta
                     write_custom_io, seek_custom_io, reinterpret_cast<intptr_t>(&io));
             if (r != GTA_OK)
             {
-                throw exception(r);
+                throw exception("cannot write GTA data block", static_cast<gta::result>(r));
             }
         }
 
@@ -2016,7 +2023,7 @@ namespace gta
                     lower_coordinates, higher_coordinates, block, f);
             if (r != GTA_OK)
             {
-                throw exception(r);
+                throw exception("cannot write GTA data block", static_cast<gta::result>(r));
             }
         }
 
@@ -2038,7 +2045,7 @@ namespace gta
                     lower_coordinates, higher_coordinates, block, fd);
             if (r != GTA_OK)
             {
-                throw exception(r);
+                throw exception("cannot write GTA data block", static_cast<gta::result>(r));
             }
         }
 
